import os
from os import path

import matplotlib.pyplot as plt
import numpy as np
import png
from scipy import fftpack
from scipy import linalg


def svd(img):
    U, d, Vt = linalg.svd(img)
    D = linalg.diagsvd(d, img.shape[0], img.shape[1])
    return U, d, D, Vt


def rank_approx(U, d, D, Vt, k):
    D = D.copy()
    D[D < d[int(k)]] = 0.
    return np.dot(np.dot(U, D), Vt)


def dct(img):
    return fftpack.dct(fftpack.dct(img.T, norm='ortho').T, norm='ortho')


def idct(dct):
    return fftpack.idct(fftpack.idct(dct.T, norm='ortho').T, norm='ortho')


def dct_approx(dct, k):
    dct_copy = dct.copy()
    dct_copy[k:, :] = 0
    dct_copy[:, k:] = 0

    r_img = idct(dct_copy)
    return r_img.clip(0, 255)


def get_path(filename):
    return 'pngs/' + filename + '.png'


def generate_sep(height):
    arr = np.empty((height, 20))
    arr.fill(0)
    return arr


def append_to_scale(scale, img):
    sep = generate_sep(len(img))
    scale.append(sep)
    scale.append(img)
    scale.append(sep)


def write_png(filename, img=None, scale=None):
    if scale:
        img = np.concatenate(scale, axis=1)

    rows = len(img)
    cols = len(img[0])

    png_writer = png.Writer(cols, rows, greyscale=True)

    if not path.isdir('pngs'):
        os.mkdir('pngs')

    f = open(get_path(filename), 'wb')
    png_writer.write(f, img)


def write_graph(graph_arr, title, filename):
    size_arr = []
    k_arr = []

    orig_size = np.double(os.stat(get_path('orig')).st_size)

    for img in graph_arr:
        size_arr.append(1 - np.divide(os.stat(get_path(img['name'])).st_size, orig_size))
        k_arr.append(img['k'])

    plt.clf()
    plt.title(title + ' K vs Compression Ratio')

    plt.xlabel('Value of K')
    plt.ylabel('Compression Ratio')

    plt.plot(k_arr, size_arr)
    for xy in zip(k_arr, size_arr):  # <--
        plt.annotate('(%.5s, %.5s)' % xy, xy=xy, textcoords='data')  #

    plt.grid()
    plt.savefig(get_path(filename))


def write_comp_graph(svd_arr, dct_arr):
    svd_size_arr = []
    dct_size_arr = []

    orig_size = np.double(os.stat(get_path('orig')).st_size)

    for img in svd_arr:
        svd_size_arr.append(os.stat(get_path(img['name'])).st_size)

    for img in dct_arr:
        dct_size_arr.append(os.stat(get_path(img['name'])).st_size)

    plt.clf()
    plt.title("SVD compression vs DCT compression")

    plt.xlabel("SVD file size")
    plt.ylabel("DCT file size")

    plt.plot(svd_size_arr, dct_size_arr)
    plt.savefig(get_path("comp_graph"))
